"""This code shows how to use pulldom with a very simple, "hand-coded"
dispatcher and a few helper functions to do an X->Y translation.

It is deliberately coded in a manner that is not as intelligent as it
could be because I want to emphasize that there is no rocket science.
It's simple enough to use in a simple manner and can easily be ramped
up to something more advanced with sophisticated dispatchers (which any
sophisticated Python programmer could write in half an hour or so)."""


import pulldom
import sys

paper= \
"""<paper>
<front>
<title>From Markup To Object Model</title>
<subt>The XML Abstraction Problem and XML Property Objects</subt>
<author>
<fname>Paul</fname><surname>Prescod</surname>
<jobtitle>Consulting Engineer</jobtitle>
<address><affil>ISOGEN/DataChannel</affil>
<aline>2200 North Lamar</aline>
<city>Dallas</city><state>Texas</state><cntry>USA</cntry><postcode>75202</postcode>
<phone>214 953 0004</phone><fax>214 953 3152</fax>
<email>paul@isogen.com</email>
<web>www.isogen.com</web>
</address>
<bio>
<para><highlight style="bold">Paul Prescod</highlight> 
- Paul Prescod is a researcher and implementor of markup
  technologies.  His formal education was in mathematics and computer
science at the <highlight style="italic">University of Waterloo</highlight>.
His research interests include
formalisms for document modeling, queries and schemata.  As a consulting
engineer at ISOGEN, he helps organizations apply ISO and W3C standards
to large-scale documentation problems.</para>
</bio>
</author>
<abstract>
<para>Mechanisms for building abstractions over XML documents tend to be
more complex and less flexible than techniques available in domains such
as relational databases and object models. This paper reviews several
existing strategies and suggests a new one. XML Property Objects allow a
flexible, user-defined mapping from complex XML attributed element tree
structures to directed labeled graph structures.</para>
</abstract>
</front>
<body>
<section>
<title>Overview</title>
<para>Software engineering is dominated by two tasks. The first is the
design of algorithms (and necessary data structures) required to
automate the solutions to particular problems. The second is the design
of abstractions. Abstractions allow us to reuse software code and thus
make software solutions that can grow and be maintained over
time.</para>

<para>In a world where only implementation and algorithms mattered,
everything could be programmed in assembly language and every project
would be approached as if from scratch. There would be no operating
systems, no programming languages, no code libraries and no relational
databases.  A programmer's job would be analogous to that of a
carpenter. The fact that a carpenter has hammered a nail a thousand
times before does not remove the requirement to do it again.  Reuse is
at the level of ideas and skills, not implementation. </para>

<para>To some extent, creators of tiny "embedded systems" live in this
world.  Thankfully, the rest of us can use the ever-expanding RAM in our
computers to build abstractions on top of abstractions on top of
abstractions: programs on top of programming languages on top of
interpreters on top of other programming languages on top of operating
systems. Each level can itself be decomposed into many abstractions. The
popular UML diagramming standard exists precisely to help manage these
levels of abstraction.</para>

<para>...</para>
</section>
</body>
</paper>
"""

output="""<html>
From Markup To Object Model
The XML Abstraction Problem and XML Property Objects
<p>Job Title: Consulting Engineer</p>
<address>
<p>ISOGEN/DataChannel</p>
<p>2200 North Lamar</p>
<p>Dallas, Texas</p>
<p>75202</p>
<p>Phone: 214 953 0004</p>
<p>Fax: 214 953 3152</p>
<p>Email: paul@isogen.com</p>
<p>Web: www.isogen.com</p>
</address>
<blockquote>
<p>
<b>
Paul Prescod
</b>
- Paul Prescod is a researcher and implementor of markup
  technologies.  His formal education was in mathematics and computer
science at the 
<i>
University of Waterloo
</i>
.
His research interests include
formalisms for document modeling, queries and schemata.  As a consulting
engineer at ISOGEN, he helps organizations apply ISO and W3C standards
to large-scale documentation problems.
</p>
</blockquote>
<p>By: Paul  Prescod
<blockquote>
<p>
Mechanisms for building abstractions over XML documents tend to be
more complex and less flexible than techniques available in domains such
as relational databases and object models. This paper reviews several
existing strategies and suggests a new one. XML Property Objects allow a
flexible, user-defined mapping from complex XML attributed element tree
structures to directed labeled graph structures.
</p>
</blockquote>
<h1>Overview</h1>
<p>
Software engineering is dominated by two tasks. The first is the
design of algorithms (and necessary data structures) required to
automate the solutions to particular problems. The second is the design
of abstractions. Abstractions allow us to reuse software code and thus
make software solutions that can grow and be maintained over
time.
</p>
<p>
In a world where only implementation and algorithms mattered,
everything could be programmed in assembly language and every project
would be approached as if from scratch. There would be no operating
systems, no programming languages, no code libraries and no relational
databases.  A programmer's job would be analogous to that of a
carpenter. The fact that a carpenter has hammered a nail a thousand
times before does not remove the requirement to do it again.  Reuse is
at the level of ideas and skills, not implementation. 
</p>
<p>
To some extent, creators of tiny "embedded systems" live in this
world.  Thankfully, the rest of us can use the ever-expanding RAM in our
computers to build abstractions on top of abstractions on top of
abstractions: programs on top of programming languages on top of
interpreters on top of other programming languages on top of operating
systems. Each level can itself be decomposed into many abstractions. The
popular UML diagramming standard exists precisely to help manage these
levels of abstraction.
</p>
<p>
...
</p>
</html>
"""


def process_paper( events ):
    for action, node in events:
        if matchStart( "paper", action, node ):
            print "<html>"
            
        elif matchEnd( "paper", action, node ):
            print "</html>"
    
        elif matchTextIn( ("title", "paper"), action, node ):
            print "<title>%s</title>" % node.data
    
        elif matchEnd( "author", action, node ):
            print "<p>By: %s %s" % ( firstname, lastname )
    
        elif matchTextIn( "fname", action, node ):
            firstname=node.data +" "
    
        elif matchTextIn( "surname", action, node ):
            lastname=node.data
    
        elif matchTextIn( "jobtitle", action, node ):
            print "<p>Job Title: %s</p>" % node.data
    
        elif matchTextIn( "affil", action, node ):
            affil=node.data
    
        elif matchTextIn( "aline", action, node ):
            aline=node.data
    
        elif matchTextIn( "city", action, node ):
            city=node.data
    
        elif matchTextIn( "state", action, node ):
            state=node.data
    
        elif matchTextIn( "cntry", action, node ):
            cntry=node.data
    
        elif matchTextIn( "postcode", action, node ):
            postcode=node.data
    
        elif matchTextIn( "phone", action, node ):
            phone=node.data
    
        elif matchTextIn( "fax", action, node ):
            fax=node.data
    
        elif matchTextIn( "email", action, node ):
            email=node.data
    
        elif matchTextIn( "web", action, node ):
            web=node.data
    
        elif matchEnd( "address", action, node ):
            print "<address>"
            print "<p>%s</p>"%affil
            print "<p>%s</p>"%aline
            print "<p>%s, %s</p>"% (city, state)
            print "<p>%s</p>"%postcode
            print "<p>Phone: %s</p>"%phone
            print "<p>Fax: %s</p>"%fax
            print "<p>Email: %s</p>"%email
            print "<p>Web: %s</p>"%web
            print "</address>"
    
        elif matchStart( "para", action, node ):
            print "<p>"
    
        elif matchEnd( "para", action, node ):
            print "</p>"
    
        elif matchStart( "highlight", action, node ):
            if node.getAttribute( "style" )=="bold":
                print "<b>"
            elif node.getAttribute( "style" )=="italic":
                print "<i>"
 
        elif matchEnd( "highlight", action, node ):
            if node.attributes["style"]=="bold":
                print "</b>"
            elif node.attributes["style"]=="italic":
                print "</i>"
    
        elif matchStart( "bio", action, node ):
            print "<blockquote>"
    
        elif matchEnd( "bio", action, node ):
            print "</blockquote>"
    
        elif matchStart( "abstract", action, node ):
            print "<blockquote>"
    
        elif matchEnd( "abstract", action, node ):
            print "</blockquote>"

        # I could have counted on the way down,
        #      but I want to show code that walks up the tree
        elif matchTextIn( ("title", "section" ), action, node ): 
            level=0
            titleNode=node.parentNode
            sectionNode=titleNode.parentNode
            while sectionNode.tagName=="section":
                level=level+1
                sectionNode=sectionNode.parentNode
            outtag="h"+`level`
            print "<%s>%s</%s>" % (outtag, node.data, outtag )
        elif action==pulldom.CHARACTERS:
            if( node.data.strip()):
                print node.data
        else: 
            pass


# a few simple helper functions
def matchStart( tagName, action, node ):
    return action==pulldom.START_ELEMENT and node.tagName==tagName

def matchEnd( tagName, action, node ):
    return action==pulldom.END_ELEMENT and node.tagName==tagName

def matchTextIn( tagName, action, node ):
    if type( tagName )==type( "" ):
        return action==pulldom.CHARACTERS and node.parentNode.tagName==tagName
    elif type( tagName ) == type((1,)):
        return action==pulldom.CHARACTERS and \
           matchTagContext( tagName, node.parentNode )

def matchTagContext( tagNames, node ):
    this,rest=tagNames[0], tagNames[1:]
    imatch = node.tagName==this  
    if not rest:
        return imatch
    else:
        return imatch and matchTagContext( rest, node.parentNode )


correct_actions = ['START_DOCUMENT', 'START_ELEMENT', 'CHARACTERS',
'START_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT',
'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS',
'START_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT',
'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT',
'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'START_ELEMENT',
'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS',
'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT',
'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'START_ELEMENT', 'CHARACTERS',
'END_ELEMENT', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS',
'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'START_ELEMENT', 'CHARACTERS',
'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT',
'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS',
'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'START_ELEMENT',
'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS',
'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'START_ELEMENT',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS',
'END_ELEMENT', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'START_ELEMENT',
'CHARACTERS', 'START_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS',
'START_ELEMENT', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'CHARACTERS',
'START_ELEMENT', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'CHARACTERS', 'START_ELEMENT',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS', 'CHARACTERS',
'END_ELEMENT', 'CHARACTERS', 'CHARACTERS', 'START_ELEMENT', 'CHARACTERS',
'END_ELEMENT', 'CHARACTERS', 'END_ELEMENT', 'CHARACTERS', 'END_ELEMENT',
'CHARACTERS', 'END_ELEMENT']


def test_paper():
    import StringIO
    eventgen=pulldom.parseString( paper )
    events=[]
    for obj in eventgen:
        events.append( obj )

    sys.stdout,ostdout=StringIO.StringIO(),sys.stdout
    process_paper( events )
    sys.stdout,ostdout=ostdout,sys.stdout
    actions=map( lambda x: (x[0], getattr( x[1], "tagName", None ) ), events )
    #open( "test1.html.out", "w" ).write( output )
    assert ostdout.getvalue()==open( "test1.html.out", "r").read() 
    assert `actions`==open( "test1.tuples.out", "r" ).read()
    sys.stdout=ostdout

test_paper()

nspaper="""<paper>
<front>
<title>From Markup To Object Model</title>
<subt>The XML Abstraction Problem and XML Property Objects</subt>
<author>
<fname>Paul</fname><surname>Prescod</surname>
<jobtitle>Consulting Engineer</jobtitle>
<address><affil>ISOGEN/DataChannel</affil>
<aline>2200 North Lamar</aline>
<city>Dallas</city><state>Texas</state><cntry>USA</cntry><postcode>75202</postcode>
<phone>214 953 0004</phone><fax>214 953 3152</fax>
<email>paul@isogen.com</email>
<web>www.isogen.com</web>
</address>
<bio>
<para><highlight style="bold">Paul Prescod</highlight> 
- Paul Prescod is a leading researcher and implementor of markup
  technologies.  His formal education was in mathematics and computer
science at the University of Waterloo. His research interests include
formalisms for document modeling, queries and schemata.  As a consulting
engineer at ISOGEN, he helps organizations apply ISO and W3C standards
to large-scale documentation problems.</para>
</bio>
</author>
<abstract>
<para>Mechanisms for building abstractions over XML documents tend to be
more complex and less flexible than techniques available in domains such
as relational databases and object models. This paper reviews several
existing strategies and suggests a new one. XML Property Objects allow a
flexible, user-defined mapping from complex XML attributed element tree
structures to directed labeled graph structures.</para>
</abstract>
</front>
<body>
<section>
<title>Overview</title>
<para>Software engineering is dominated by two tasks. The first is the
design of algorithms (and necessary data structures) required to
automate the solutions to particular problems. The second is the design
of abstractions. Abstractions allow us to reuse software code and thus
make software solutions that can grow and be maintained over
time.</para>

<para>In a world where only implementation and algorithms mattered,
everything could be programmed in assembly language and every project
would be approached as if from scratch. There would be no operating
systems, no programming languages, no code libraries and no relational
databases.  A programmer's job would be analogous to that of a
carpenter. The fact that a carpenter has hammered a nail a thousand
times before does not remove the requirement to do it again.  Reuse is
at the level of ideas and skills, not implementation. </para>

<para>To some extent, creators of tiny "embedded systems" live in this
world.  Thankfully, the rest of us can use the ever-expanding RAM in our
computers to build abstractions on top of abstractions on top of
abstractions: programs on top of programming languages on top of
interpreters on top of other programming languages on top of operating
systems. Each level can itself be decomposed into many abstractions. The
popular UML diagramming standard exists precisely to help manage these
levels of abstraction.</para>

<para>...</para>
</section>
</body>
</paper>
"""

